Rem
	Audio streaming with 'Raylib Audio' Example
	2019 A.Woodard "GW"
Endrem

SuperStrict
Framework brl.basic 
Import brl.glmax2d

Import "C:\Dev\temp\Raylib\src\raudio.c"

Extern
	Function IsAudioDeviceReady%()
	Function InitAudioDevice()
	Function CloseAudioDevice()
	Function IsAudioBufferProcessed:Int(stream:AudioStream)                '// Check If any audio stream buffers requires refill
	Function PlayAudioStream(stream:AudioStream)                        '// Play audio stream
	Function InitAudioStream:AudioStream(sampleRate:UInt, sampleSize:UInt, channels:UInt)            '// Init audio stream (To stream raw audio pcm data)
	Function UpdateAudioStream(stream:AudioStream, Data:Byte Ptr, samplesCount:Int) '// Update audio stream buffers with data
	Function SetMasterVolume(volume:Float)
EndExtern

Struct AudioStream
    Field sampleRate:UInt      '// Frequency (samples per second)
    Field sampleSize:UInt      '// Bit depth (bits per sample): 8, 16, 32 (24 Not supported)
    Field channels:UInt        '// Number of channels (1-mono, 2-stereo)
    Field audioBuffer:Byte Ptr '// Pointer To internal data used by the audio system.
    Field format%              '// Audio format specifier
    Field source%         '// Audio source id
    Field buffers%[2]     '// Audio buffers (Double buffering)
End Struct

'--------------------------------START-----------------------------------------------
AppTitle = "RayLib realtime audio streaming"
Graphics 1024, 500
SetLineWidth(1.5)

InitAudioDevice()

If Not IsAudioDeviceReady()
	RuntimeError("No Audio")
EndIf

OnEnd(CloseAudioDevice)

Const BUFFERSIZE:Int = 2048
Global mystream:AudioStream = InitAudioStream(44100, 32, 1) ;	'// Create the audio stream 
Global a_writebuff:Float[BUFFERSIZE]						'// create a buffer and a pointer to the buffer
Global writeBuf:Float Ptr = VarPtr a_writebuff[0]
Global Time:Float = 0

Const TWOPI:Double = Pi * 2


PlayAudioStream(mystream)	'// Start playing

SetMasterVolume(0.25)	'// easy on the ears

Print "Starting!"

Const FRQ:Float = 107.66
Global phase:Float

While True
	If KeyHit(KEY_ESCAPE) Then End
	Local buffproc:Int = IsAudioBufferProcessed(mystream)	'// if the sound card wants more data this is true
	If buffproc Then 
		For Local i:Int = 0 Until BUFFERSIZE
			'// Get waveform
			phase:+FRQ * (TWOPI / mystream.samplerate)
			If phase > Pi Then phase:-TWOPI
			Local x:Float = sw(FRQ, phase)	'calc audio data 
			'//Mod w/mouse
			x:*Float(MouseY()) / GraphicsHeight()	'// change volume by MouseY
			Local yy:Float = Float(MouseX()) / GraphicsWidth()	'// change gain by MouseX
			If Abs(x * (yy)) > 0.5 Then x = Sine(-x)	'//fold distortion
			
			'//write to buffer
			a_writebuff[i] = x * 0.25
		Next
		UpdateAudioStream(mystream, writeBuf, BUFFERSIZE)	'// tell raylib to use the new buffer
		Draw
	EndIf
Wend

'---------------------------------------------------------------------------------------------------------------------------	 
Function Draw()
	Local y:Float
	Local y2:Float
	Local t:Int = 0
	For Local i:Int = 0 Until BUFFERSIZE Step 2
		y = y2
		y2 = 250 + (a_writebuff[i] * 500)
		DrawLine(i / 2, y, i / 2, y2)
	Next
	Flip
	Cls
	Time:+1
End Function
'---------------------------------------------------------------------------------------------------------------------------
Function Sine:Double(x:Double)
	Return Sin(x / (Pi / 180.0))
End Function
'---------------------------------------------------------------------------------------------------------------------------
Function sw:Float(f:Float, ph:Float)
	Local i:Float = 1, x:Float
	While (f * i) < (4000 * (1.1 + Sin(Time * 4)))
		x:+(Sine(ph * i) * (1.0 / i))
		i:+1 + Abs(Sine(Time / 10))
	Wend
	Return x
End Function
'---------------------------------------------------------------------------------------------------------------------------